"use strict";

/* eslint-disable no-console */

import transform from 'css-to-react-native';
const DEBUG = true;
function log(...args) {
  if (DEBUG) {
    console.log('[StyleUtils]', ...args);
  }
}
const ALLOWED_STYLE_PROPERTIES = {
  color: true,
  background: true,
  backgroundColor: true,
  fontWeight: true,
  fontStyle: true,
  textDecoration: true
};

// Map VSCode font styles to RN styles
const FONT_STYLE_MAP = {
  bold: {
    fontWeight: 'bold'
  },
  italic: {
    fontStyle: 'italic'
  },
  underline: {
    textDecorationLine: 'underline'
  }
};
export function getStylesFromShikiTheme(theme) {
  log('Processing theme:', {
    name: typeof theme === 'string' ? theme : theme.name,
    hasSettings: !!theme.settings,
    settingsCount: theme.settings?.length,
    hasTokenColors: !!theme.tokenColors?.length
  });
  const styles = {
    root: {
      backgroundColor: theme.bg || theme.colors?.['editor.background'] || '#000000',
      color: theme.fg || theme.colors?.['editor.foreground'] || '#ffffff'
    }
  };
  log('Initial root styles:', styles.root);

  // Handle both VSCode theme format and Shiki theme format
  const tokenSettings = theme.settings || theme.tokenColors || [];
  tokenSettings.forEach((setting, index) => {
    if (!setting.scope && !setting.settings) return;
    const scopes = Array.isArray(setting.scope) ? setting.scope : typeof setting.scope === 'string' ? setting.scope.split(',').map(s => s.trim()) : [];
    if (scopes.length === 0) {
      // Global styles
      if (setting.settings) {
        const processedStyles = processTokenSettings(setting.settings);
        if (processedStyles) {
          styles.root = {
            ...styles.root,
            ...processedStyles
          };
        }
      }
      return;
    }
    scopes.forEach(scope => {
      if (!scope) return;
      const rawStyle = processTokenSettings(setting.settings);
      if (rawStyle) {
        styles[scope] = rawStyle;
      }
      if (DEBUG && index === 0) {
        log('First theme setting conversion:', {
          scope,
          settings: setting.settings,
          rawStyle,
          finalStyle: styles[scope]
        });
      }
    });
  });

  // Process specific token types that might not be in the theme
  const defaultTokenStyles = {
    keyword: {
      color: '#f97583'
    },
    string: {
      color: '#9ecbff'
    },
    comment: {
      color: '#6a737d'
    },
    variable: {
      color: '#e1e4e8'
    },
    number: {
      color: '#79b8ff'
    },
    function: {
      color: '#b392f0'
    }
  };
  Object.entries(defaultTokenStyles).forEach(([scope, style]) => {
    if (!styles[scope]) {
      styles[scope] = style;
    }
  });
  log('Generated styles for scopes:', Object.keys(styles));
  return styles;
}
function processTokenSettings(settings) {
  const style = {};
  if (settings.foreground) {
    style.color = settings.foreground;
  }
  if (settings.fontStyle) {
    const fontStyles = settings.fontStyle.split(' ');
    fontStyles.forEach(fs => {
      const mappedStyle = FONT_STYLE_MAP[fs];
      if (mappedStyle) {
        Object.assign(style, mappedStyle);
      }
    });
  }
  if (settings.background) {
    style.backgroundColor = settings.background;
  }
  return cleanStyle(style);
}
export function cleanStyle(style) {
  const filteredEntries = Object.entries(style).filter(([key, value]) => ALLOWED_STYLE_PROPERTIES[key] && value !== undefined).map(([key, value]) => [key, value]);
  if (DEBUG) {
    log('Cleaning style:', {
      input: style,
      filtered: filteredEntries
    });
  }
  return transform(filteredEntries);
}
//# sourceMappingURL=style.js.map