/* eslint-disable no-console */
import type { ThemeRegistrationAny } from '@shikijs/types'
import type { TextStyle } from 'react-native'
import transform, { type StyleTuple } from 'css-to-react-native'

const DEBUG = true
function log(...args: any[]) {
  if (DEBUG) {
    console.log('[StyleUtils]', ...args)
  }
}

export interface ShikiStyleSheet {
  [key: string]: TextStyle
}

interface TokenSettings {
  foreground?: string
  background?: string
  fontStyle?: string
  settings?: {
    foreground?: string
    background?: string
    fontStyle?: string
  }
}

interface TokenColor {
  scope?: string | string[]
  settings: TokenSettings
}

const ALLOWED_STYLE_PROPERTIES: Record<string, boolean> = {
  color: true,
  background: true,
  backgroundColor: true,
  fontWeight: true,
  fontStyle: true,
  textDecoration: true,
}

// Map VSCode font styles to RN styles
const FONT_STYLE_MAP: Record<string, Partial<TextStyle>> = {
  bold: { fontWeight: 'bold' },
  italic: { fontStyle: 'italic' },
  underline: { textDecorationLine: 'underline' },
}

export function getStylesFromShikiTheme(theme: ThemeRegistrationAny): ShikiStyleSheet {
  log('Processing theme:', {
    name: typeof theme === 'string' ? theme : theme.name,
    hasSettings: !!theme.settings,
    settingsCount: theme.settings?.length,
    hasTokenColors: !!(theme as any).tokenColors?.length,
  })

  const styles: ShikiStyleSheet = {
    root: {
      backgroundColor: theme.bg || theme.colors?.['editor.background'] || '#000000',
      color: theme.fg || theme.colors?.['editor.foreground'] || '#ffffff',
    },
  }

  log('Initial root styles:', styles.root)

  // Handle both VSCode theme format and Shiki theme format
  const tokenSettings = (theme.settings || (theme as any).tokenColors || []) as TokenColor[]

  tokenSettings.forEach((setting: TokenColor, index: number) => {
    if (!setting.scope && !setting.settings)
      return

    const scopes = Array.isArray(setting.scope) ? setting.scope : typeof setting.scope === 'string' ? setting.scope.split(',').map((s: string) => s.trim()) : []

    if (scopes.length === 0) {
      // Global styles
      if (setting.settings) {
        const processedStyles = processTokenSettings(setting.settings)
        if (processedStyles) {
          styles.root = { ...styles.root, ...processedStyles }
        }
      }
      return
    }

    scopes.forEach((scope: string) => {
      if (!scope)
        return

      const rawStyle = processTokenSettings(setting.settings)
      if (rawStyle) {
        styles[scope] = rawStyle
      }

      if (DEBUG && index === 0) {
        log('First theme setting conversion:', {
          scope,
          settings: setting.settings,
          rawStyle,
          finalStyle: styles[scope],
        })
      }
    })
  })

  // Process specific token types that might not be in the theme
  const defaultTokenStyles: Record<string, TextStyle> = {
    keyword: { color: '#f97583' },
    string: { color: '#9ecbff' },
    comment: { color: '#6a737d' },
    variable: { color: '#e1e4e8' },
    number: { color: '#79b8ff' },
    function: { color: '#b392f0' },
  }

  Object.entries(defaultTokenStyles).forEach(([scope, style]) => {
    if (!styles[scope]) {
      styles[scope] = style
    }
  })

  log('Generated styles for scopes:', Object.keys(styles))
  return styles
}

function processTokenSettings(settings: TokenSettings): TextStyle {
  const style: Record<string, any> = {}

  if (settings.foreground) {
    style.color = settings.foreground
  }

  if (settings.fontStyle) {
    const fontStyles = settings.fontStyle.split(' ')
    fontStyles.forEach((fs: string) => {
      const mappedStyle = FONT_STYLE_MAP[fs]
      if (mappedStyle) {
        Object.assign(style, mappedStyle)
      }
    })
  }

  if (settings.background) {
    style.backgroundColor = settings.background
  }

  return cleanStyle(style)
}

export function cleanStyle(style: Record<string, any>): TextStyle {
  const filteredEntries = Object.entries(style)
    .filter(([key, value]) => ALLOWED_STYLE_PROPERTIES[key] && value !== undefined)
    .map<StyleTuple>(([key, value]) => [key, value])

  if (DEBUG) {
    log('Cleaning style:', {
      input: style,
      filtered: filteredEntries,
    })
  }

  return transform(filteredEntries)
}
